# Copyright 2020 John Reese
# Licensed under the MIT License

"""
What's the deal with asyncio?
"""

__author__ = "John Reese"
from .__version__ import __version__
from .seinfeld import Seinfeld
from .types import (
    Episode,
    Episodes,
    Speaker,
    Speakers,
    Season,
    Seasons,
    Quote,
    Passage,
    Name,
)
